//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <glib.h>

#include "net.h"

static gboolean server_io(GIOChannel *gio, GIOCondition condition, gpointer data);
static GIOChannel *gio_server;
#define PACKET_BUFF_SIZE 128

//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
static gboolean
send_packet_timeout(gpointer data)
{
    static gchar *func_name = "send_packet_timer";
    char buff[128];
    int offset = 0;

    memset(buff, 0, sizeof(buff));
    offset = 0;


    // short frame
    buff[offset++] = 0x10; // start
    buff[offset++] = 0x5B; // control field: init slave
    buff[offset++] = 0xFE; // address field
    buff[offset++] = 0x5B + 0xFE; // checksum
    buff[offset++] = 0x16; // end

/*
    // from manual:
    // Set the slave to primary address 8 without changing anything else:
    buff[offset++] = 0x68;
    buff[offset++] = 0x06;
    buff[offset++] = 0x06;
    buff[offset++] = 0x68;
    buff[offset++] = 0x53;
    buff[offset++] = 0xFE;
    buff[offset++] = 0x51;
    buff[offset++] = 0x01;
    buff[offset++] = 0x7A;
    buff[offset++] = 0x08;
    buff[offset++] = 0x25;
    buff[offset++] = 0x16;
*/

    if (fs_net_send(gio_server, buff, offset) < offset)
    { 
        g_message("%s: failed to write data to server", func_name);
    }    
	
    return TRUE; 
}

//------------------------------------------------------------------------------
// client_io:
//
//------------------------------------------------------------------------------
static gboolean
server_io(GIOChannel *gio, GIOCondition condition, gpointer data)
{
    static gchar *func_name = "server_io";
    gchar buff[PACKET_BUFF_SIZE];
    gint len;
    int fd, i;
    FsNetBuffer *nbdata;

    bzero((void *)buff, sizeof(buff));

    if ((fd = g_io_channel_unix_get_fd(gio)) < 0)
    {
       g_warning("%s: Failed to aquire socket fd.", func_name);
       return FALSE;
    }

    if (condition & (G_IO_HUP|G_IO_NVAL|G_IO_ERR))
    {
        exit(0);
    }

    if ((nbdata  = fs_net_buffer_get(gio)) == NULL)
    {
        g_warning("%s: Serious error, couldn't retrieve data buffer for GIO Channel.", func_name);
        return FALSE;
    }


    if ((len = fs_net_read_data_nonblocking(gio, nbdata, 100)) < 0)
    {
        g_message("%s: read returned (len = %d) \n", func_name, len);

        if (len == -2 && nbdata->offset == 0)
        {
            // Return value -2 is a special return value that we use to indicate
            // that the read operation was interrupted due to unavailability of 
            // data (so we got an try-again error, preventing a blocking read).
            //
            // Restart the read operation later.
            return TRUE;
        }        
	
	    if (len == -1)
	    {
	        g_message("%s: Closing socket, out of data (%d).", func_name, fd);
                fs_net_buffer_free(gio);
	        fs_net_socket_close(fd);
                exit(0);
        }
    }
    len = nbdata->offset;

    g_message("%s: received %d bytes from the server", func_name, len);
    for (i = 0; i < len; i++)
    {
        printf("nbdata[%d] = 0x%.2X\n", i, ((int)nbdata->buff[i] & 0xFF));
    }

    exit(0);	
    return TRUE;
}

//------------------------------------------------------------------------------
// Execution starts here:
//
//------------------------------------------------------------------------------
int
main(int argc, char **argv)
{
    static gchar *func_name = "mbus_test1", *host;
    int port;

    GMainLoop *gloop;
 
    if (argc != 3)
    {
        g_message("%s: usage: %s host port", func_name, argv[0]);
        return 0;
    }
 
    host = argv[1];   
    port = atoi(argv[2]);
    
    if ((gio_server = fs_net_connection_setup(host, port, (void *)server_io, NULL)) == NULL)
    {
        g_warning("%s: Failed to setup connection to server.\n", func_name);
        return -1;
    }

    // Register new data buffer for the new GIO Channel.
    if (fs_net_buffer_new(gio_server) != 0)
    {
        g_warning("%s: Failed to allocate GIO channel data buffer.", __PRETTY_FUNCTION__);
        return NULL;
    }
   	
    /* setup a timer */
    g_timeout_add(1000, send_packet_timeout, NULL);
   	
    /* Pass over flow control to GLIB */
    gloop = g_main_loop_new(NULL, FALSE);
    g_main_loop_run(gloop);

    /* Never reached */
    return 0;
}



